@@ -41,6 +41,7 @@ int analyze_format(FILE *f) {
 	buffer[4]=0;
 	if (strncmp(buffer,write_sign,2)==0) {
 		printf("[Windows Write file. Some garbage expected]\n");
+		get_unicode_char=get_8bit_char;
 		return process_file(f,LONG_MAX);
 	} else if (strncmp(buffer,rtf_sign,4)==0) {
 		return parse_rtf(f);
@@ -48,20 +49,26 @@ int analyze_format(FILE *f) {
 	   fread(buffer+4,1,124,f);	
 	   return parse_word_header(buffer,f,128,0);
 	}	
-	if ((new_file=ole_init(f, buffer, 4)) != NULL) {
-		set_ole_func();
-		while((ole_file=ole_readdir(new_file)) != NULL) {
-			int res=ole_open(ole_file);
-			if (res >= 0) {
-				if (strcmp(((oleEntry*)ole_file)->name , "WordDocument") == 0) {
-					offset=catdoc_read(buffer, 1, 128, ole_file);
-					ret_code=parse_word_header(buffer,ole_file,-offset,offset);
-				}
-			} 
-			ole_close(ole_file);
-		}
-		set_std_func();
-		ole_finish();
+	fread(buffer+4,1,4,f);
+	if (strncmp(buffer,ole_sign,8)==0) {
+		if ((new_file=ole_init(f, buffer, 8)) != NULL) {
+			set_ole_func();
+			while((ole_file=ole_readdir(new_file)) != NULL) {
+				int res=ole_open(ole_file);
+				if (res >= 0) {
+					if (strcmp(((oleEntry*)ole_file)->name , "WordDocument") == 0) {
+						offset=catdoc_read(buffer, 1, 128, ole_file);
+						ret_code=parse_word_header(buffer,ole_file,-offset,offset);
+					}
+				} 
+				ole_close(ole_file);
+			}
+			set_std_func();
+			ole_finish();
+		} else {
+			fprintf(stderr,"Broken OLE file. Try using -b switch");
+			exit(1);
+		}	
 	} else {
 		copy_out(f,buffer);
 		return 0;
@@ -107,7 +114,7 @@ int parse_word_header(unsigned char * buffer,FILE *f,int offset,long curpos) {
 	if (flags & fComplex) {
 		fprintf(stderr,"[This was fast-saved %2d times. Some information is lost]\n",
 				(flags & 0xF0)>>4);
-		ret_code=69;
+/*		ret_code=69;*/
 	}
 	if (verbose) {
 		if (flags & fReadOnly) {
