@@ -13,6 +13,7 @@
 #include "xls.h"
 #include "catdoc.h"
 #include "xltypes.h"
+#include "float.h"
 #include <math.h>
 #include <time.h>
 #ifndef HAVE_STRFTIME
@@ -21,9 +22,13 @@
 static unsigned char rec[MAX_MS_RECSIZE];
 int biff_version=0;
 short int *formatTable=NULL;
+char *forced_date_format = NULL;
 size_t formatTableIndex = 0;
 size_t formatTableSize = 0;
 
+#define FLT_FORMAT(a,b,c) a #b c
+#define MK_FORMAT(x) FLT_FORMAT("%.",x,"g") 			
+char number_format[8]=MK_FORMAT(DBL_DIG);
 void do_table(FILE *input,char *filename) {    
 	long rectype;
 	long reclen,build_year=0,build_rel=0,offset=0;
@@ -231,6 +236,8 @@ void process_item (int rectype, int reclen, char *rec) {
 									  fprintf(stderr,"CONSTANT_STRING before SST parsed\n");
 									  exit(1);
 								  }    
+/* 									fprintf(stderr,"col=%d row=%d no=%d\n",col,row,string_no); */
+									
 								  saved_reference=NULL;
 								  pcell=allocate(row,col);
 								  if (string_no>=sstsize|| string_no < 0 ) {
@@ -413,6 +420,12 @@ char *copy_unicode_string (unsigned char **src) {
 		count=**src;
 		flags = *(*src+offset);
 		offset --;
+		flags = *(*src+1+offset);
+		if (! ( flags == 0 || flags == 1 || flags == 8 || flags == 9 ||
+						flags == 4 || flags == 5 || flags == 0x0c || flags == 0x0d ) ) {
+/* 			fprintf(stderr,"Strange flags = %d, returning NULL\n", flags); */
+			return NULL;
+		}
 	}
 	else {
 		count=getshort(*src,0);
@@ -444,7 +457,8 @@ char *copy_unicode_string (unsigned char **src) {
 			/* 		fprintf(stderr,"Default string\n"); */
 	}
 
-	/* 	fprintf(stderr,"count=%d skip=%d start_offset=%d\n",count, to_skip, start_offset); */
+/* 	fprintf(stderr,"count=%d skip=%d start_offset=%d\n", */
+/* 					count, to_skip, start_offset); */
 	/*     	*/
 	if ( (dest=malloc(count+1)) == NULL ) {
 		perror("Dest string alloc error");
@@ -524,11 +538,14 @@ static char *formats[]={
 	}
 	index = formatTable[format_code];
 	if ((index>= 0x0E) && (index<=0x16)) {
-		return formats[index-0x0E];
+		if (forced_date_format) return forced_date_format;
+			return formats[index-0x0E];
 	} else 	
 	if ((index>=0x2d) && (index<=0x2F)) {
+		if (forced_date_format) return forced_date_format;
 		return formats[index-0x2d+9];
 	} else if (index==0xa4) {	
+		if (forced_date_format) return forced_date_format;
 		return formats[12];
 	} else 	
 	return NULL;
@@ -544,8 +561,8 @@ char *number2string(double d,short int format_code) {
 		if ((datefmt=isDateFormat(format_code))!=NULL) {
 		   time_t t = float2date(d);	
 		   strftime(buffer, 127,datefmt, gmtime(&t));  
-		} else {	
-		 	sprintf(buffer,"%.10g",d);
+		} else {
+		 	sprintf(buffer,number_format,d);
 		}
 		return buffer;
 }
@@ -591,15 +608,15 @@ char* format_rk(char *rec,short int format_code) {
 			for(i=0;i<8;i++)
 				dconv.cc[i]='\0';
 # ifdef WORDS_BIGENDIAN     
-			for(s=rec+4,d=dconv.cc,i=0;
-					i<4;i++) *(d++)=*(--s);
-					dconv.cc[0]=dconv.cc[0] & 0xfc;
+			for(s=rec+4,d=dconv.cc,i=0; i<4;i++) 
+				*(d++)=*(--s);
+			dconv.cc[0]=dconv.cc[0] & 0xfc;
 # else       
-					for(s=rec,d=dconv.cc+4,i=0;
+			for(s=rec,d=dconv.cc+4,i=0;
 							i<4;i++) *(d++)=*(s++);
-							dconv.cc[4]=dconv.cc[4] & 0xfc;
-							value=dconv.d;
+							dconv.cc[3]=dconv.cc[3] & 0xfc;
 # endif     
+			value=dconv.d;
 	}
 	if ( *(rec) & 0x01 ) 
 		value=value*0.01;
@@ -634,9 +651,12 @@ void parse_sst(char *sstbuf,int bufsize) {
 		perror("SST allocation error");
 		exit(1);
 	}
+	memset(sst,0,sstsize*sizeof(char *));
 	for (i=0,parsedString=sst,curString=sstbuf+8;
 			i<sstsize && curString<barrier; i++,parsedString++) {
-
+/* 		fprintf(stderr,"copying %d string\n",i); */
 		*parsedString = copy_unicode_string(&curString);
 	}       
+/* 	fprintf(stderr,"end sst i=%d sstsize=%d\n",i,sstsize); */
+
 }	
